import os

from elasticsearch import Elasticsearch


class ElasticSearch:
    def __init__(self):
        self.client = Elasticsearch(
            [os.getenv("ELASTICSEARCH_HOST", "localhost:9200/")]
        )



import os

import requests


def serve_request_write_content(url, destination):
    r = requests.get(url, allow_redirects=True)
    open(destination + "/" + url.split("/")[-1], "wb").write(r.content)


def download_file(source, destination, multiple=False):
    os.makedirs(destination, exist_ok=True)
    if multiple:
        for url in source:
            serve_request_write_content(url=url, destination=destination)
    else:
        serve_request_write_content(url=source, destination=destination)



from json import loads, dumps

from rest_framework import exceptions, status
from rest_framework.exceptions import APIException


def make_keys_to_dot(d):
    """
    Flattens the dictionary containing other dictionaries like here:
    https://stackoverflow.com/questions/6027558/flatten-nested-python-dictionaries-compressing-keys
    """

    def expand(key, value):
        if isinstance(value, dict):
            return [(key + "." + k, v) for k, v in make_keys_to_dot(value).items()]
        else:
            return [(key, value)]

    items = [item for k, v in d.items() for item in expand(k, v)]
    return dict(items)


def does_data_exist_in_generator(doc):
    """
    :param doc: generator obj
    :return: flag
    """
    for _data in doc:
        yield _data
        return True
    return False


def nested_ordered_dict_to_dict(input_ordered_dict):
    return loads(dumps(input_ordered_dict))


def record_not_found_error(message):
    """
    :param message: the message to be printed
    """
    raise exceptions.NotFound(message)


class RequestBodyNotAcceptable(APIException):
    status_code = status.HTTP_422_UNPROCESSABLE_ENTITY
    default_detail = "Unprocessable Entity"
    default_code = "unprocessable_entity"


def is_generator_empty(generator):
    try:
        item = next(generator)

        def my_generator():
            yield item
            yield from generator
        return my_generator(), False
    except StopIteration:
        return (_ for _ in []), True



from django.db import connection


def execute_select_query(query, fetch=None):
    data = None
    c = connection.cursor()
    try:
        c.execute(query)
        if fetch and fetch == "one":
            data = c.fetchone()
        else:
            data = c.fetchall()
    finally:
        c.close()
        return data


def execute_update_query(query):
    c = connection.cursor()
    try:
        c.execute(query)
    finally:
        c.close()



from rest_framework import status
from rest_framework.exceptions import APIException


class TriggerInternalServerError(APIException):
    status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
    default_detail = "Internal Server Error"
    default_code = "internal_server_error"
